# odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
# odl_clientId = "qja9jy5wnjyqwvylmeqw"
# uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"
# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"
# baseUrl: https://sso.staging.openxlab.org.cn/gw/uaa-be

# TODO toggle this when publishing
# production
# odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
# staging
# odl_clientId= 'ypkl8bwo0eb5ao1b96no'


# endpoint = "http://staging.openxlab.org.cn"

# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"
# uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"

FILE_THRESHOLD = 30 * 1024 * 1024
# FILE_THRESHOLD = 100


# # staging
# odl_clientId = 'ypkl8bwo0eb5ao1b96no'
# endpoint = "https://staging.openxlab.org.cn"
# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"

# # prod
odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
endpoint = "https://openxlab.org.cn"
uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"


computed_url = "/datasets/api/v2/"
