from openxlab.model.handler.download_file import download as download, download_metafile_template as init, download_from_url as wget
from openxlab.model.handler.create_repository import create
from openxlab.model.handler.update_repository import visibility
from openxlab.model.handler.remove_repo_or_file import remove
from openxlab.model.handler.query_repo_model import list
from openxlab.model.handler.upload_file import upload
from openxlab.model.handler.model_inference import Inference
from openxlab.model.handler.model_inference import inference


